@@ -81,13 +81,21 @@ buildCommand bs = intercalate " && " (go (getBuilder bs))
 	go Cabal =
 		[ "cabal configure"
 		, "cabal build -j1 propellor-config"
-		, "ln -sf dist/build/propellor-config/propellor-config propellor"
+		, "ln -sf" `commandCabalBuildTo` "propellor"
 		]
 	go Stack =
 		[ "stack build :propellor-config"
 		, "ln -sf $(stack path --dist-dir)/build/propellor-config/propellor-config propellor"
 		]
 
+commandCabalBuildTo :: ShellCommand -> FilePath -> ShellCommand
+commandCabalBuildTo cmd dest = intercalate "; "
+	[ "if [ -d dist-newstyle ]"
+	, "then " ++ cmd ++ " $(cabal exec -- sh -c 'command -v propellor-config') " ++ shellEscape dest
+	, "else " ++ cmd ++ " dist/build/propellor-config/propellor-config " ++ shellEscape dest
+	, "fi"
+	]
+
 -- Check if all dependencies are installed; if not, run the depsCommand.
 checkDepsCommand :: Bootstrapper -> Maybe System -> ShellCommand
 checkDepsCommand bs sys = go (getBuilder bs)
@@ -95,7 +103,7 @@ checkDepsCommand bs sys = go (getBuilder bs)
 	go Cabal = "if ! cabal configure >/dev/null 2>&1; then " ++ depsCommand bs sys ++ "; fi"
 	go Stack = "if ! stack build --dry-run >/dev/null 2>&1; then " ++ depsCommand bs sys ++ "; fi"
 
-data Dep = Dep String | OldDep String
+data Dep = Dep String | OptDep String | OldDep String
 
 -- Install build dependencies of propellor, using the specified
 -- Bootstrapper.
@@ -131,7 +139,8 @@ depsCommand bs msys = "( " ++ intercalate " ; " (go bs) ++ ") || true"
 	useapt builder = "apt-get update" : map aptinstall (debdeps builder)
 
 	aptinstall (Dep p) = "DEBIAN_FRONTEND=noninteractive apt-get -qq --no-upgrade --no-install-recommends -y install " ++ p
-	aptinstall (OldDep p) = "if LANG=C apt-cache policy " ++ p ++ "| grep -q Candidate:; then " ++ aptinstall (Dep p) ++ "; fi"
+	aptinstall (OptDep p) = "if LANG=C apt-cache policy " ++ p ++ "| grep -q Candidate:; then " ++ aptinstall (Dep p) ++ "; fi"
+	aptinstall (OldDep p) = aptinstall (OptDep p)
 	pkginstall p = "ASSUME_ALWAYS_YES=yes pkg install " ++ p
 	pacmaninstall p = "pacman -S --noconfirm --needed " ++ p
 
@@ -152,6 +161,8 @@ depsCommand bs msys = "( " ++ intercalate " ; " (go bs) ++ ") || true"
 		, Dep "libghc-exceptions-dev"
 		, Dep "libghc-text-dev"
 		, Dep "libghc-hashable-dev"
+		-- Deps that can be skipped on old systems.
+		, OptDep "libghc-type-errors-dev"
 		-- Deps that are only needed on old systems.
 		, OldDep "libghc-stm-dev"
 		]
@@ -257,32 +268,28 @@ buildPropellor mh = unlessM (actionMessage "Propellor build" build) $
 -- dependencies and retries.
 cabalBuild :: Maybe System -> IO Bool
 cabalBuild msys = do
-	make "dist/setup-config" ["propellor.cabal"] cabal_configure
+	make "configured" ["propellor.cabal"] cabal_configure
 	unlessM cabal_build $
 		unlessM (cabal_configure <&&> cabal_build) $
 			error "cabal build failed"
-	-- For safety against eg power loss in the middle of the build,
-	-- make a copy of the binary, and move it into place atomically.
-	-- This ensures that the propellor symlink only ever points at
-	-- a binary that is fully built. Also, avoid ever removing
-	-- or breaking the symlink.
-	--
-	-- Need cp -pfRL to make build timestamp checking work.
-	unlessM (boolSystem "cp" [Param "-pfRL", Param cabalbuiltbin, Param (tmpfor safetycopy)]) $
+	-- Make a copy of the binary, and move it into place atomically.
+	let safetycopy = "propellor.built"
+	let cpcmd = "cp -pfL" `commandCabalBuildTo` safetycopy
+	unlessM (boolSystem "sh" [Param "-c", Param cpcmd]) $
 		error "cp of binary failed"
-	rename (tmpfor safetycopy) safetycopy
-	symlinkPropellorBin safetycopy
+	rename safetycopy "propellor"
 	return True
   where
-	cabalbuiltbin = "dist/build/propellor-config/propellor-config"
-	safetycopy = cabalbuiltbin ++ ".built"
 	cabal_configure = ifM (cabal ["configure"])
-		( return True
+		( do
+			writeFile "configured" ""
+			return True
 		, case msys of
 			Nothing -> return False
 			Just sys ->
 				boolSystem "sh" [Param "-c", Param (depsCommand (Robustly Cabal) (Just sys))]
 					<&&> cabal ["configure"]
+					<&&> (writeFile "configured" "" >> return True)
 		)
 	-- The -j1 is to only run one job at a time -- in some situations,
 	-- eg in qemu, ghc does not run reliably in parallel.
