@@ -2,7 +2,6 @@ module Propellor.Debug where
 
 import Control.Monad.IfElse
 import System.IO
-import System.Directory
 import System.Log.Logger
 import System.Log.Formatter
 import System.Log.Handler (setFormatter)
@@ -14,6 +13,7 @@ import Utility.Monad
 import Utility.Env
 import Utility.Exception
 import Utility.Process
+import Utility.Directory
 
 debug :: [String] -> IO ()
 debug = debugM "propellor" . unwords
