@@ -97,86 +97,6 @@ scrollBox = propertyList "scroll server" $ props
 	s = d </> "login.sh"
 	g = d </> "game.sh"
 
-oldUseNetServer :: [Host] -> Property (HasInfo + DebianLike)
-oldUseNetServer hosts = propertyList "olduse.net server" $ props
-	& Apt.installed ["leafnode"]
-	& oldUseNetInstalled "oldusenet-server"
-	& oldUseNetBackup
-	& spoolsymlink
-	& "/etc/news/leafnode/config" `File.hasContent`
-		[ "# olduse.net configuration (deployed by propellor)"
-		, "expire = 1000000" -- no expiry via texpire
-		, "server = " -- no upstream server
-		, "debugmode = 1"
-		, "allowSTRANGERS = 42" -- lets anyone connect
-		, "nopost = 1" -- no new posting (just gather them)
-		]
-	& "/etc/hosts.deny" `File.lacksLine` "leafnode: ALL"
-	& Apt.serviceInstalledRunning "openbsd-inetd"
-	& File.notPresent "/etc/cron.daily/leafnode"
-	& File.notPresent "/etc/cron.d/leafnode"
-	& Cron.niceJob "oldusenet-expire" (Cron.Times "11 1 * * *") (User "news") newsspool expirecommand
-	& Cron.niceJob "oldusenet-uucp" (Cron.Times "*/5 * * * *") (User "news") "/" uucpcommand
-	& Apache.siteEnabled "nntp.olduse.net" nntpcfg
-  where
-	newsspool = "/var/spool/news"
-	datadir = "/var/spool/oldusenet"
-	expirecommand = intercalate ";"
-		[ "find \\( -path ./out.going -or -path ./interesting.groups -or -path './*/.overview' \\) -prune -or -type f -ctime +60  -print | xargs --no-run-if-empty rm"
-		, "find -type d -empty | xargs --no-run-if-empty rmdir"
-		]
-	uucpcommand = "/usr/bin/uucp " ++ datadir
-	nntpcfg = apachecfg "nntp.olduse.net"
-		[ "  DocumentRoot " ++ datadir ++ "/"
-		, "  <Directory " ++ datadir ++ "/>"
-		, "    Options Indexes FollowSymlinks"
-		, "    AllowOverride None"
-		, Apache.allowAll
-		, "  </Directory>"
-		]
-
-	spoolsymlink :: Property UnixLike
-	spoolsymlink = check (not . isSymbolicLink <$> getSymbolicLinkStatus newsspool)
-		(property "olduse.net spool in place" $ makeChange $ do
-			removeDirectoryRecursive newsspool
-			createSymbolicLink (datadir </> "news") newsspool
-		)
-
-	oldUseNetBackup :: Property (HasInfo + DebianLike)
-	oldUseNetBackup = Borg.backup datadir borgrepo 
-		(Cron.Times "33 4 * * *")
-		[]
-		[Borg.KeepDays 30]
-		`requires` Ssh.userKeyAt (Just keyfile)
-			(User "root")
-			(Context "olduse.net")
-			(SshEd25519, "ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIGtHhNuO/abNTrc5pyWkwXz/F8FAHw8VcET0Yh4mjgOk root@kite")
-		`requires` Ssh.knownHost hosts "usw-s002.rsync.net" (User "root")
-	borgrepo = rsyncNetBorgRepo "olduse.net.borg" [Borg.UseSshKey keyfile]
-	keyfile = "/root/.ssh/olduse.net.key"
-
-oldUseNetShellBox :: Property DebianLike
-oldUseNetShellBox = propertyList "olduse.net shellbox" $ props
-	& oldUseNetInstalled "oldusenet"
-	& Service.running "shellinabox"
-
-oldUseNetInstalled :: Apt.Package -> Property DebianLike
-oldUseNetInstalled pkg = check (not <$> Apt.isInstalled pkg) $
-	propertyList ("olduse.net " ++ pkg) $ props
-		& Apt.installed (words "build-essential devscripts debhelper git libncursesw5-dev libpcre3-dev pkg-config bison libicu-dev libidn11-dev libcanlock2-dev libuu-dev ghc libghc-ifelse-dev libghc-hxt-dev libghc-utf8-string-dev libghc-missingh-dev libghc-sha-dev haskell-stack")
-			`describe` "olduse.net build deps"
-		& scriptProperty
-			[ "rm -rf /root/tmp/oldusenet" -- idenpotency
-			, "git clone git://olduse.net/ /root/tmp/oldusenet/source"
-			, "cd /root/tmp/oldusenet/source/"
-			, "HOME=/root dpkg-buildpackage -us -uc"
-			, "dpkg -i ../" ++ pkg ++ "_*.deb || true"
-			, "apt-get -fy install" -- dependencies
-			, "rm -rf /root/tmp/oldusenet"
-			]
-			`assume` MadeChange
-			`describe` "olduse.net built"
-
 kgbServer :: Property (HasInfo + DebianLike)
 kgbServer = propertyList desc $ props
 	& Apt.serviceInstalledRunning "kgb-bot"
@@ -328,7 +248,7 @@ gitAnnexDistributor = combineProperties "git-annex distributor, including rsync
 	-- git-annex distribution signing key
 	& Gpg.keyImported (Gpg.GpgKeyId "89C809CB") (User "joey")
 	-- used for building rpms
-	& Apt.installed ["rpm", "createrepo"]
+	& Apt.installed ["rpm", "createrepo-c"]
   where
 	endpoint d = combineProperties ("endpoint " ++ d) $ props
 		& File.dirExists d
@@ -439,7 +359,7 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 	& Apt.serviceInstalledRunning "clamav-freshclam"
 	-- Workaround https://bugs.debian.org/569150
 	& Cron.niceJob "amavis-expire" Cron.Daily (User "root") "/"
-		"find /var/lib/amavis/virusmails/ -type f -ctime +7 -delete"
+		"find /var/lib/amavis/virusmails/ -type f -ctime +2 -delete"
 
 	& dkimInstalled
 
@@ -600,7 +520,10 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 		`onChange` (imapalpinescript `File.mode`
 			combineModes (readModes ++ executeModes))
 		`describe` "imap script for pine"
-	& Apt.serviceInstalledRunning "mailman"
+	-- XXX temporarily disabled installing as it's not available in
+	-- debian unstable any longer. Need to upgrade to mailman3
+	-- at some point. (nontrivial)
+	-- & Apt.serviceInstalledRunning "mailman"
 	-- Override the default http url. (Only affects new lists.)
 	& "/etc/mailman/mm_cfg.py" `File.containsLine`
 		"DEFAULT_URL_PATTERN = 'https://%s/cgi-bin/mailman/'"
@@ -1145,7 +1068,7 @@ ipmasq intif = File.hasContent ifupscript
 laptopSoftware :: Property DebianLike
 laptopSoftware = Apt.installed
 	[ "intel-microcode", "acpi"
-	, "procmeter3", "xfce4", "procmeter3", "unclutter"
+	, "procmeter3", "xfce4", "procmeter3", "unclutter-xfixes"
 	, "mplayer", "fbreader", "firefox", "chromium"
 	, "libdatetime-event-sunrise-perl", "libtime-duration-perl"
 	, "network-manager", "network-manager-openvpn-gnome", "openvpn"
@@ -1167,6 +1090,7 @@ laptopSoftware = Apt.installed
 	, "vorbis-tools", "audacity"
 	, "ekiga"
 	, "bluez-firmware", "blueman", "pulseaudio-module-bluetooth"
+	, "fwupd"
 	, "xul-ext-ublock-origin", "xul-ext-pdf.js", "xul-ext-status4evar"
 	, "vim-syntastic", "vim-fugitive"
 	, "adb", "gthumb"
@@ -1254,6 +1178,10 @@ homeNAS = propertyList "home NAS" $ props
 		(USBHubPort hubvendor 4)
 		(USBDriveId wd "25a3")
 		(Just "archive")
+	& autoMountDrivePort "archive-14"
+		(USBHubPort hubvendor 2)
+		(USBDriveId wd "25a3")
+		Nothing
 	& autoMountDrive "passport" Nothing
 	& Apt.installed ["git-annex", "borgbackup"]
   where
@@ -1280,7 +1208,7 @@ autoMountDrivePort :: Mount.Label -> USBHubPort -> USBDriveId -> Maybe FilePath
 autoMountDrivePort label hp drive malias = propertyList desc $ props
 	& File.hasContent ("/etc/systemd/system/" ++ hub)
 		[ "[Unit]"
-		, "Description=Startech usb hub port " ++ show (hubPort hp)
+		, "Description=Startech usb hub port " ++ show (hubPort hp) ++ " vendor " ++ driveVendorId drive ++ " driveid " ++ driveProductId drive
 		, "PartOf=" ++ mount
 		, "[Service]"
 		, "Type=oneshot"
@@ -1307,7 +1235,7 @@ autoMountDrivePort label hp drive malias = propertyList desc $ props
 	devfile = "/dev/disk/by-label/" ++ label
 	mountpoint = "/media/joey/" ++ label
 	desc = "auto mount with hub port power control " ++ mountpoint
-	hub = "startech-hub-port-" ++ show (hubPort hp) ++ ".service"
+	hub = "startech-hub-port-" ++ show (hubPort hp) ++ "-vendor-" ++ driveVendorId drive ++ "-drivedid-" ++ driveProductId drive ++ ".service"
 	mount = svcbase ++ ".mount"
 	svcbase = Systemd.escapePath mountpoint
 	selecthubport = unwords
